@@ -5,7 +5,7 @@ config.update("jax_enable_x64", True)
 # Highlight any potentially unintended broadcasting rank promoting ops.
 # config.update("jax_numpy_rank_promotion", "warn")
 
-from .abstractions import batch_loader, fit, fit_batches, optax_fit
+from .abstractions import fit, fit_batches
 from .gps import Prior, construct_posterior
 from .kernels import (
     RBF,
@@ -22,6 +22,6 @@ from .mean_functions import Constant, Zero
 from .parameters import copy_dict_structure, initialise, transform
 from .sparse_gps import SVGP
 from .types import Dataset
-from .variational import VariationalGaussian
+from .variational import VariationalGaussian, WhitenedVariationalGaussian
 
-__version__ = "0.4.3"
+__version__ = "0.4.5"
